;; Copyright 2006, 2007, 2009, 2010, 2013, 2015 Kevin Ryde

;; This file is part of accjournal.
;;
;; accjournal is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; accjournal is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with accjournal; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))
(require 'accjournal)

(defun my-run-test ()
  (interactive)
  (let ((count 0))

    ;;-------------------------------------------------------------------------
    ;; check autoloads

    ;; emacs24 `generate-file-autoloads' wants to be in a file-visiting
    ;; buffer
    (require 'autoload)
    (find-file "t-accjournal--generated-loaddefs.el")
    (erase-buffer)
    (generate-file-autoloads "../accjournal.el")
    (eval-buffer)
    (save-buffer)
    (kill-buffer (current-buffer))
    (delete-file "t-accjournal--generated-loaddefs.el")

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(accjournal-mode))

    ;;-------------------------------------------------------------------------
    ;; accjournal-add

    (let* ((try (lambda (func x y want)
                  (let ((got  (condition-case err
                                  (funcall func x y)
                                (error nil)))) ;; (error-message-string err)
                    (unless (equal got want)
                      (error "`%S %S %S got %S want %S"
                             func x y got want)))
                  (setq count (1+ count))))
           (try2 (lambda (func x y want)
                   (funcall try func x y want)
                   (funcall try func y x want))))

      (funcall try2 'accjournal-add most-positive-fixnum  0   most-positive-fixnum)
      (funcall try2 'accjournal-add most-positive-fixnum  1   nil)
      (funcall try2 'accjournal-add most-positive-fixnum -1   (1- most-positive-fixnum))
      (funcall try2 'accjournal-add most-positive-fixnum most-positive-fixnum  nil)
      (funcall try2 'accjournal-add most-positive-fixnum most-negative-fixnum
               (+ most-positive-fixnum most-negative-fixnum))

      (funcall try2 'accjournal-add most-negative-fixnum  0   most-negative-fixnum)
      (funcall try2 'accjournal-add most-negative-fixnum  1   (1+ most-negative-fixnum))
      (funcall try2 'accjournal-add most-negative-fixnum -1   nil)
      (funcall try2 'accjournal-add most-negative-fixnum most-positive-fixnum
               (+ most-positive-fixnum most-negative-fixnum))
      (funcall try2 'accjournal-add most-negative-fixnum most-negative-fixnum nil)

      ;; sub

      (funcall try 'accjournal-sub most-negative-fixnum  0   most-negative-fixnum)
      (funcall try 'accjournal-sub most-negative-fixnum  1   nil)
      (funcall try 'accjournal-sub most-negative-fixnum -1   (1+ most-negative-fixnum))
      (funcall try 'accjournal-sub most-negative-fixnum most-positive-fixnum  nil)
      (funcall try 'accjournal-sub most-positive-fixnum most-negative-fixnum  nil)
      (funcall try 'accjournal-sub most-negative-fixnum most-negative-fixnum
               0)
      (funcall try 'accjournal-sub most-positive-fixnum most-positive-fixnum
               0))


    ;;-------------------------------------------------------------------------
    ;; safe-local-variable-p

    (if (not (fboundp 'safe-local-variable-p))
        (message "skip, no `safe-local-variable-p'")

      (unless (safe-local-variable-p 'accjournal-mandatory-open t)
        (error "accjournal-mandatory-open=t not safe-local-variable-p"))
      (setq count (1+ count))

      (unless (safe-local-variable-p 'accjournal-mandatory-open nil)
        (error "accjournal-mandatory-open=t not safe-local-variable-p"))
      (setq count (1+ count))

      (when (safe-local-variable-p 'accjournal-mandatory-open "blah")
        (error "accjournal-mandatory-open=string is safe-local-variable-p"))
      (setq count (1+ count)))


    ;;-------------------------------------------------------------------------
    ;; customize

    (save-excursion
      (customize-group 'accjournal)
      (kill-buffer nil))
    (setq count (1+ count))

    (if (not (fboundp 'custom-group-of-mode))
        (message "skip, no `custom-group-of-mode' in this emacs")
      (let ((group (custom-group-of-mode 'accjournal-mode)))
        (unless (equal group 'accjournal)
          (error "custom-group-of-mode is %S expected `accjournal'" group)))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------
    ;; local variables

    (with-temp-buffer
      (set (make-local-variable 'my-run-test-local-variable) 123)
      (unless (boundp 'my-run-test-local-variable)
        (error "oops, my-run-test-local-variable not bound"))

      (accjournal-mode)

      (when (boundp 'my-run-test-local-variable)
        (error "my-run-test-local-variable still bound"))
      (setq count (1+ count))

      (unless (equal major-mode 'accjournal-mode)
        (error "major-mode is %S" major-mode))
      (setq count (1+ count))

      (unless (equal mode-name "AccJournal")
        (error "mode-name is %S" mode-name))
      (setq count (1+ count))

      (unless (equal (current-local-map) accjournal-mode-map)
        (error "(current-local-map) is not accjournal-mode-map"))
      (setq count (1+ count))

      (unless (equal (syntax-table) accjournal-mode-syntax-table)
        (error "(syntax-table) is not accjournal-syntax-table"))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------
    ;; hooks run

    (let ((my-hook-count nil)
          (my-hook-func (lambda ()
                          (setq my-hook-count (1+ my-hook-count)))))

      ;; accjournal-mode-hook
      (with-temp-buffer
        (add-hook 'accjournal-mode-hook my-hook-func)
        (setq my-hook-count 0)
        (accjournal-mode)
        (remove-hook 'accjournal-mode-hook my-hook-func))
      (unless (equal my-hook-count 1)
        (error "Oops, accjournal-mode-hook count %S" my-hook-count))
      (setq count (1+ count))


      ;; after-change-major-mode-hook, if available
      (if (not (boundp 'after-change-major-mode-hook))
          (message "no after-change-major-mode-hook in this emacs")
        (with-temp-buffer
          (add-hook 'after-change-major-mode-hook my-hook-func)
          (setq my-hook-count 0)
          (accjournal-mode)
          (remove-hook 'after-change-major-mode-hook my-hook-func))
        (unless (equal my-hook-count 1)
          (error "Oops, after-change-major-mode-hook count %S" my-hook-count))
        (setq count (1+ count))))


    ;;-------------------------------------------------------------------------
    ;; accjournal-Mmm-to-month

    (mapcar (lambda (elem)
              (let* ((str  (nth 0 elem))
                     (want (nth 1 elem))
                     (got  (condition-case err
                               (accjournal-Mmm-to-month str)
                             (error (error-message-string err)))))
                (unless (equal got want)
                  (error "`accjournal-Mmm-to-month' str %S got %S want %S"
                         str got want)))
              (setq count (1+ count)))

            '(("Jun" 6)
              ("JUN" 6)
              ("jun" 6)
              ("Aug" 8)
              ("" "Unrecognised month: \"\"")))

    ;;-------------------------------------------------------------------------
    ;; accjournal-year

    (mapcar (lambda (elem)
              (let* ((str (nth 0 elem))
                     (want (nth 1 elem))
                     (got (accjournal-year str)))
                (unless (equal got want)
                  (error "`accjournal-year' str %S got %S want %S"
                         str got want)))
              (setq count (1+ count)))

            '(("1995" 1995)
              ("2010" 2010)))

    (if (let ((this-year (nth 5 (decode-time))))
          (message "year %S" this-year)
          (or (> this-year 2030)
              (< this-year 2000)))
        (message "skip, year out of range for fixed tests")

      (mapcar (lambda (elem)
                (let* ((str (nth 0 elem))
                       (want (nth 1 elem))
                       (got (accjournal-year str)))
                  (unless (equal got want)
                    (error "`accjournal-year' str %S got %S want %S"
                           str got want)))
                (setq count (1+ count)))

              '(("95" 1995)
                ("10" 2010))))

    ;;-------------------------------------------------------------------------
    ;; accjournal-date-to-N

    (let ((accjournal-year-base (- 2010 60)))
      (mapcar (lambda (elem)
                (let* ((str (nth 0 elem))
                       (want (nth 1 elem))
                       (got (condition-case err
                                (accjournal-date-to-N str)
                              (error (error-message-string err)))))
                  (unless (equal got want)
                    (error "`accjournal-date-to-N' str %S got %S want %S"
                           str got want)))
                (setq count (1+ count)))

              '(("2001-02-03" 20010203)
                ("99-02-03"   "Unrecognised date: \"99-02-03\"")
                ("1 Apr 10" 20100401)
                (" 1 Apr 10" 20100401)
                ("31 Dec 10" 20101231))))

    ;;-------------------------------------------------------------------------
    ;; accjournal-cents-to-string

    (mapcar (lambda (elem)
              (let* ((accjournal-decimal-point  (nth 0 elem))
                     (accjournal-decimal-places (nth 1 elem))
                     (n    (nth 2 elem))
                     (want (nth 3 elem))
                     (got  (accjournal-cents-to-string n)))
                (unless (equal got want)
                  (error "`accjournal-cents-to-string' places=%S n=%S got %S want %S"
                         accjournal-decimal-places n got want)))
              (setq count (1+ count)))

            '(("." 2     0   "0.00")
              ("." 2     1   "0.01")
              ("." 2    -1  "-0.01")
              ("." 2    51   "0.51")
              ("." 2   -51  "-0.51")
              ("." 2  -123  "-1.23")
              ("." 2   123   "1.23")
              ("." 2  -123  "-1.23")
              ("." 2  1234  "12.34")
              ("." 2 -1234 "-12.34")

              ("," 1     0    "0,0")
              ("," 1     1    "0,1")
              ("," 1    -1   "-0,1")
              ("," 1    51    "5,1")
              ("," 1   -51   "-5,1")
              ("," 1  -123  "-12,3")
              ("," 1   123   "12,3")
              ("," 1  -123  "-12,3")
              ("," 1  1234  "123,4")
              ("," 1 -1234 "-123,4")

              ("." 0     0     "0")
              ("." 0     1     "1")
              ("." 0    -1    "-1")
              ("." 0    51    "51")
              ("." 0   -51   "-51")
              ("." 0  -123  "-123")
              ("." 0   123   "123")
              ("." 0  -123  "-123")
              ("." 0  1234  "1234")
              ("." 0 -1234 "-1234")))

    ;;-------------------------------------------------------------------------
    ;; accjournal-amount-regexp

    (mapcar (lambda (elem)
              (let* ((str  (nth 0 elem))
                     (want (nth 1 elem))
                     (got  (string-match (concat "\\`"
                                                 accjournal-amount-regexp
                                                 "\\'")
                                         str)))
                (unless (equal got want)
                  (error "`accjournal-amount-regexp' str %S got %S want %S"
                         str got want)))
              (setq count (1+ count)))

            '(("0" 0)
              ("x" nil)
              ("-x" nil)
              ("-" nil)
              ("-1-" nil)
              ("1.2.3" nil)
              ("1,000.00" nil)
              ("," nil)
              ("." nil)

              ("12" 0)
              ("-12" 0)

              ("1.2" 0)
              ("1,2" 0)
              ("1.20" 0)
              ("1,20" 0)

              (".20" 0)
              (",20" 0)

              ("1." 0)
              ("1," 0)
              ("123." 0)
              ("123," 0)))

    ;;-------------------------------------------------------------------------
    ;; accjournal-line-regexp

    (mapcar (lambda (elem)
              (let* ((str  (car elem))
                     (want (cdr elem))
                     (got  (and (string-match accjournal-line-regexp str)
                                (list (match-string 1 str) ;; date
                                      (match-string 2 str) ;; amount
                                      (match-string 6 str) ;; from open
                                      (match-string 7 str) ;; from name
                                      (match-string 8 str) ;; from close
                                      (match-string 9 str)  ;; to open 
                                      (match-string 10 str) ;; to name 
                                      (match-string 11 str) ;; to close
                                      (match-string 12 str))))) ;; note
                (unless (equal got want)
                  (error "`accjournal-amount-regexp' str %S\ngot  %S\nwant %S"
                         str got want)))
              (setq count (1+ count)))

            '((" 8 Sep 15  100.00  Bank -> Cash | note"
               " 8 Sep 15" "100.00"
               nil "Bank" nil
               nil "Cash" nil
               "note")

              (" 8 Sep 15  100.00  Bank -> Cash |"
               " 8 Sep 15" "100.00"
               nil "Bank" nil
               nil "Cash" nil
               "")

              (" 8 Sep 15  100.00  Bank -> Cash"
               " 8 Sep 15" "100.00"
               nil "Bank" nil
               nil "Cash" nil
               nil)

              ;; trailing whitespace on to-name
              (" 8 Sep 15  100.00  Bank -> Cash  "
               " 8 Sep 15" "100.00"
               nil "Bank" nil
               nil "Cash" nil
               nil)))

    ;;-------------------------------------------------------------------------

    (message "  %d tests" count)
    count))





;;     ;;-------------------------------------------------------------------------
;;     ;; accjournal-string-to-cents
;; 
;;     (with-temp-buffer
;;       (accjournal-make-alist))
;;     (mapcar (lambda (elem)
;;               (let* ((str  (nth 0 elem))
;;                      (want (nth 1 elem))
;;                      (got  (condition-case err
;;                                (accjournal-string-to-cents str)
;;                              (error
;;                               (format "error: %s"
;;                                       (error-message-string err))))))
;;                 (unless (equal got want)
;;                   (error "`accjournal-string-to-cents' str %S got %S want %S"
;;                          str got want)))
;;               (setq count (1+ count)))
;; 
;;             '(("-1.23" -123)
;;               ("18.79" 1879)
;;               (  ".01"    1)
;;               ( "0.01"    1)
;;               ("1.234" "error: Bad money amount 1.234")))
